// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "GUITest.h"
#include "ChildView.h"
#include "ObjectDlg.h"
#include <highgui.h>
#include <cv.h>
#include <calibinit\dmcalibinit.h>
#include <math.h>
#include <vector>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView()
{
  m_SrcImage.Create(10, 10, 8);
  m_ResImage.Create(10, 10, 8);

  m_iHeight = 5;
  m_iWidth  = 6;
}

CChildView::~CChildView()
{
  m_SrcImage.Destroy();
  m_ResImage.Destroy();
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_COMMAND(ID_FILE_OPENIMAGE, OnFileOpenimage)
	ON_COMMAND(ID_DETECT_CALIBRATIONOBJECTPARAM, OnDetectCalibrationObjectParam)
	ON_COMMAND(ID_DETECT_NEWMETHOD, OnDetectNewMethod)
	ON_COMMAND(ID_DETECT_OLDOPENCVMETHOD, OnDetectOldopenCVMethod)
	ON_COMMAND(ID_FILE_SAVERESULT, OnFileSaveresult)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);

	return TRUE;
}


//===================================================================
//= Function name	: CChildView::OnPaint
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
  m_ResImage.Show(dc.GetSafeHdc(), 0, 0, m_ResImage.Width(), m_ResImage.Height());
	// TODO: Add your message handler code here
	
	// Do not call CWnd::OnPaint() for painting messages
}



//===================================================================
//= Function name	: CChildView::OnFileOpenimage
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnFileOpenimage() 
{
	CFileDialog Dlg(true, NULL, NULL, OFN_FILEMUSTEXIST, "Image files regions (*.bmp, *jpg)|*.bmp;*jpg||");
	
  if (Dlg.DoModal() == IDOK)
  {
    if (m_SrcImage.Load(Dlg.GetPathName(), 0))
    {
      m_ResImage.CopyOf(m_SrcImage);
    }
    else
      MessageBox("Error loading file", "File error", MB_OK | MB_ICONERROR);
  }

  Invalidate();
}


//===================================================================
//= Function name	: CChildView::OnDetectCalibrationObjectParam
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnDetectCalibrationObjectParam() 
{
	CObjectDlg Dlg;

  Dlg.m_iWidth  = m_iWidth;
  Dlg.m_iHeight = m_iHeight;


  if (Dlg.DoModal() == IDOK)
  {
    m_iWidth = Dlg.m_iWidth;
    m_iHeight = Dlg.m_iHeight;
  }	
}


//===================================================================
//= Function name	: CChildView::OnDetectNewMethod
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnDetectNewMethod() 
{
  CvvImage TempImage;
  bool     bFound;
  int      iPointCount = m_iWidth * m_iHeight * 2;
  
  std::vector <CvPoint2D32f> FoundPoints;
  FoundPoints.resize(iPointCount);
  
  TempImage.CopyOf(m_SrcImage, 0);
  
  bFound = dmFindChessBoardCornerGuesses( m_SrcImage.GetImage(), TempImage.GetImage(), NULL,
                                          cvSize( m_iWidth - 1, m_iHeight - 1),
                                          &FoundPoints[0], &iPointCount  ) != 0;
  
  m_ResImage.CopyOf(m_SrcImage, 1);

  if (bFound)
  {
    cvFindCornerSubPix(m_SrcImage.GetImage(), &FoundPoints[0], iPointCount, 
                       cvSize(5,5), cvSize(-1,-1), 
                       cvTermCriteria( CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10, 0.01f ));
                       
    DrawDetectedPoints(&FoundPoints[0], iPointCount, m_iWidth - 1);
  }

  Invalidate();
}


//===================================================================
//= Function name	: CChildView::OnDetectOldopenCVMethod
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnDetectOldopenCVMethod() 
{
  CvvImage TempImage;
  bool     bFound;
  int      iPointCount = m_iWidth * m_iHeight * 2;
  
  std::vector <CvPoint2D32f> FoundPoints;
  FoundPoints.resize(iPointCount);
  
  TempImage.CopyOf(m_SrcImage, 0);
  
  bFound = cvFindChessBoardCornerGuesses( m_SrcImage.GetImage(), TempImage.GetImage(), NULL,
                                          cvSize( m_iWidth - 1, m_iHeight - 1),
                                          &FoundPoints[0], &iPointCount ) != 0;
  
  m_ResImage.CopyOf(m_SrcImage, 1);

  if (bFound)
  {
    cvFindCornerSubPix(m_SrcImage.GetImage(), &FoundPoints[0], iPointCount, 
                       cvSize(5,5), cvSize(-1,-1), 
                       cvTermCriteria( CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10, 0.01f ));

    DrawDetectedPoints(&FoundPoints[0], iPointCount, m_iWidth - 1);
  }

  Invalidate();
}


//===================================================================
//= Function name	: CChildView::DrawDetectedPoints
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::DrawDetectedPoints(CvPoint2D32f *in_pPoints, int in_iPoints, int in_iObjWdt)
{
  if( m_ResImage.GetImage() && in_iPoints )
  {
      CvMat dst_stub, *dst;
      int count = in_iPoints;
      bool found = true;

      cvScale(m_ResImage.GetImage(), m_ResImage.GetImage(), 0.5, 64);

      dst = cvGetMat( m_ResImage.GetImage(), &dst_stub );

      static const CvScalar line_colors[] =
      {
          {0,0,255},
          {0,128,255},
          {0,200,200},
          {0,255,0},
          {200,200,0},
          {255,0,0},
          {255,0,255}
      };

      const int colorCount = sizeof(line_colors)/sizeof(line_colors[0]);
      const int r = 4;
      CvScalar color = line_colors[0];

      CvPoint prev_pt = { 0, 0};

      for(int j = 0; j < count; j++ )
      {
          CvPoint pt;
          pt.x = cvRound(in_pPoints[j].x);
          pt.y = cvRound(in_pPoints[j].y);

          if( found )
          {
              color = line_colors[(j/cvRound(in_iObjWdt))%colorCount];

              if( j != 0 )
                  cvLine( dst, prev_pt, pt, color, 1, CV_AA );
          }

          cvLine( dst,
                  cvPoint( pt.x - r, pt.y - r ),
                  cvPoint( pt.x + r, pt.y + r ), color, 1, CV_AA );

          cvLine( dst,
                  cvPoint( pt.x - r, pt.y + r),
                  cvPoint( pt.x + r, pt.y - r), color, 1, CV_AA );

          cvCircle( dst, pt, r+1, color, 1, CV_AA );

          prev_pt = pt;
      }
  }
}

//===================================================================
//= Function name	: CChildView::DrawDetectedPoints
//= Description   : 
//= Return type   : void 
//===================================================================

void CChildView::OnFileSaveresult() 
{
	CFileDialog Dlg(false, "bmp", NULL, OFN_OVERWRITEPROMPT, "Image files (*.bmp, *jpg)|*.bmp;*jpg||");
	
  if (Dlg.DoModal() == IDOK)
  {
    if (!m_ResImage.Save(Dlg.GetPathName()))
      MessageBox("Error saving file", "File error", MB_OK | MB_ICONERROR);
  }
}
